using System;
using System.Collections.Generic;
using System.IO;
using System.Threading.Tasks;
using Microsoft.Extensions.Logging;
using Props.Shop.Adafruit.Api;
using Props.Shop.Adafruit.Persistence;
using Props.Shop.Framework;
using Xunit;

namespace Props.Shop.Adafruit.Tests.Api
{
    public class LiveProductListingManagerTest
    {
        [Fact]
        public async Task CacheTest()
        {
            // TODO: Improve testability of caching system, IProductListingManager, and implement here.
            //Given
            ProductListingsParser parser = new ProductListingsParser();
            using (Stream stream = File.OpenRead("./Assets/products.json"))
            {
                parser.BuildProductListings(stream);
            }
            Dictionary<string, IList<ProductListing>> listingNames = new Dictionary<string, IList<ProductListing>>();
            foreach (ProductListing product in parser.ProductListings)
            {
                IList<ProductListing> sameProducts = listingNames.GetValueOrDefault(product.Name);
                if (sameProducts == null)
                {
                    sameProducts = new List<ProductListing>();
                    listingNames.Add(product.Name, sameProducts);
                }

                sameProducts.Add(product);
            }
            ProductListingCacheData cache = new ProductListingCacheData(listingNames);
            await Task.Delay(500);
            LiveProductListingManager mockLiveProductListingManager = new LiveProductListingManager(null, new Logger<LiveProductListingManager>(LoggerFactory.Create((builder) => builder.AddXUnit())), cache);
            //When
            mockLiveProductListingManager.RefreshProductListings();
            //Then
            Assert.True(cache.LastUpdatedUtc.Equals(mockLiveProductListingManager.LastDownload));
            Assert.NotEmpty(await mockLiveProductListingManager.ProductListings);
        }
    }
}